#include "lcd_ch.h"

void lcd_data(unsigned char);
void lcd_command(unsigned char);

void ini_ports(void);
void En_imp(void);

void wr_data(unsigned char);
unsigned char busy_flag(void);
int8_t lcd_read_AC(void);

void zob_text(char *);
void def_Clear_spec_znaky(void);

void def_znak(unsigned char *, unsigned char);
void def_spec_znaky(void);

unsigned char Znak_OO[8] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x15, 0}; // Prazdny znak	dole su tri bodky

unsigned char Znak_SC[8] = {0x1A, 0x1D, 0x04, 0x04, 0x04, 0x05, 0x02, 0}; // st.C
unsigned char Znak_SM[8] = {0x0A, 0x04, 0x0E, 0x10, 0x0E, 0x01, 0x1E, 0}; //  s + mekcen
unsigned char Znak_CM[8] = {0x0A, 0x04, 0x0E, 0x10, 0x10, 0x11, 0x0C, 0}; //  c + mekcen
	
unsigned char Znak_AD[8]= {0x02, 0x04, 0x0E, 0x01, 0x0F, 0x11, 0x0F, 0}; // a + dlzen
unsigned char Znak_TM[8]= {0x05, 0x0A, 0x08, 0x1C, 0x08, 0x09, 0x06, 0}; // t + mekcen

// Vynulovanie CGRAM
void def_Clear_spec_znaky(void)
{
	// zapis specialneho znaku do CGRAM  na poziciu 0, a 7
	for (char i = 0; i < 8; i++)
		def_znak(Znak_OO, i);
	lcd_command(0x80);
}

void def_spec_znaky(void)
{
	// zapis vlastnych znakkov do CGRAM  na poziciu 4, a 0 (8)

	def_znak(Znak_SC, 4); // stupen Celzia
	def_znak(Znak_SM, 0); // s + makcen
	def_znak(Znak_AD, 2); // a + dlzen
	def_znak(Znak_TM, 3); // t + mekcen
	// obnovenie obsahu AC. AC nastvime na 1. riadok, 1. znak
	lcd_command(0x80);
}

void lcd_cursor(bool value)
{
	lcd_command(value? 0x08:0x0C);
}

void def_spec_znaky_AC(void)
{
	// zapis specialneho znaku do CGRAM  na poziciu 7

	unsigned char obsah_AC = 0;
	obsah_AC = lcd_read_AC(); //
	// kon_vyp = obsah_AC;

	// sem dame nove vlastne znaky
	def_znak(Znak_CM, 7); // c + makcen
						  // obnovenie obsahu AC. AC

	lcd_command(0x80 | obsah_AC);
}

void ini_ports(void)
{

	/* inicializacia portov - vstupy / vystupy
	oba typy displaja */
	// nasledovna # riadky su spolocne pre oba typy LCD
	LCD_CTRL_DDR |= (1 << LCD_EN_pin);															   //  (Enable)    output
	LCD_CTRL_DDR |= (1 << LCD_RS_pin);															   //  (RS)        output
	LCD_DATA_DDR |= (1 << LCD_D4_pin) | (1 << LCD_D5_pin) | (1 << LCD_D6_pin) | (1 << LCD_D7_pin); // Datove piny ako Output (Data pre display)
	// klasicky LCD vyuziva aj RW pin
	LCD_CTRL_DDR |= (1 << LCD_RW_pin); //	(RW)        output
}

void EnOn(void)
{
	LCD_CTRL_PORT |= (1 << LCD_EN_pin); // -> "log.1"
	LCD_DELAY;							// cca 400 ns
}

void EnOff(void)
{
	LCD_CTRL_PORT &= ~(1 << LCD_EN_pin); // -> "log.0"
	LCD_DELAY;
}

void En_imp(void)
{
	LCD_CTRL_PORT |= (1 << LCD_EN_pin);	 // -> "log.1"
	LCD_DELAY;							 // cca 400 ns
	LCD_CTRL_PORT &= ~(1 << LCD_EN_pin); // -> "log.0"  spolu cca 500 ns
}

void lcd_init(void)
{				 // 4 bitove pripojenie display-a
	ini_ports(); // inicializacia porov
	_delay_ms(15);
	// 1. -------------------
	LCD_CTRL_PORT &= ~(1 << LCD_RS_pin); // set RS  = to "log. 0"  Instruction Register (IR)
	LCD_CTRL_PORT &= ~(1 << LCD_RW_pin); // set R/W = to "log. 0" - Write
	// RS R/W   DB7 DB6 DB5 DB4   DB3 DB2 DB1 DB0
	// 0  0     0   0   1   1     x   x   x   x     = 0x30
	PORT_DATA_WR_H(0x30); // 8-bitove pripojenie
	En_imp();
	// 2. -------------------
	// zopakujem  8-bitove pripojenie
	_delay_ms(5);
	En_imp();
	// 3. -------------------
	// zopakujem  8-bitove pripojenie
	_delay_ms(1);
	En_imp(); // -  stacilo by delay 0.1ms, momentalne najkratsie nastavitelne je 1ms
	// 4. -------------------
	// zmenim na 4-bitove pripojenie
	// RS R/W    DB7 DB6 DB5 DB4    DB3 DB2 DB1 DB0
	// 0  0      0   0   1   0      x   x   x   x    = 0x20
	PORT_DATA_WR_H(0x20); // 4-bitove pripojenie
	_delay_ms(1);
	En_imp(); // -  stacilo by delay 0.04ms
	// -------------------
	// LCD function set      mod: DL = 0 - 4-bitove data, N = 1 - 2 riadky, F = 0 - 5x7 dots
	// RS R/W    DB7 DB6 DB5 DB4    DB3 DB2 DB1 DB0
	// 0  0      0   0   1   DL     N   F   x   x     = 0x28
	// Mozeme nasledujuci prikaz pre LCD vynechat?
	_delay_ms(2);
	lcd_command(0x28);
	// RS R/W    DB7 DB6 DB5 DB4    DB3 DB2 DB1 DB0
	// 0  0      0   0   0   0	    0   0   0   1     = 0x01 , Display clear
	// Mozeme nasledujuci prikaz pre LCD vynechat?
	_delay_ms(2);
	lcd_command(0x01);
	// RS R/W    DB7 DB6 DB5 DB4    DB3 DB2 DB1 DB0
	// 0  0      0   0   0   0	    0   1   I/D S      = 0x06, I/D = 1 - inkrement
	// Mozeme nasledujuci prikaz pre LCD vynechat?
	_delay_ms(2);
	lcd_command(0x02);
	// RS R/W    DB7 DB6 DB5 DB4    DB3 DB2 DB1 DB0
	// 0  0      0   0   0   0	    1   D   C   B      = 0x0E, D = C = 1 - Display a kurzor zapnute
	// Mozeme nasledujuci prikaz pre LCD vynechat?
	_delay_ms(2);
	lcd_command(0x0E); // B = 0 - blikanie vypnute
}

// zapis data do Data Register
void lcd_data(unsigned char data)
{
	while (busy_flag() & 0x80)
		;
	LCD_CTRL_PORT |= (1 << LCD_RS_pin);	 //    (RS = High)
	LCD_CTRL_PORT &= ~(1 << LCD_RW_pin); //    (RW = Low, write)
	wr_data(data);
}

// zapis commandu do Instruction Register
void lcd_command(unsigned char command)
{
	while (busy_flag() & 0x80)
		;
	// while(rd_BF()); // test BF sa da realizovat pre klasicky LCD, nie Shield
	LCD_CTRL_PORT &= ~(1 << LCD_RS_pin); //   (RS = Low)
	LCD_CTRL_PORT &= ~(1 << LCD_RW_pin); //   (RW = Low, write)
	wr_data(command);
}

void wr_data(unsigned char data)
{
	PORT_DATA_WR_H(data); // data High nibble
	En_imp();

	PORT_DATA_WR_L(data); // data Low nibble
	En_imp();
}

// klasicke pripojenie LCD umozni aj test BF
unsigned char busy_flag(void)
{ //  rd_BF
	unsigned char pom = 0;
	LCD_CTRL_PORT &= ~(1 << LCD_RS_pin);															  //   (RS = Low)
	LCD_CTRL_PORT |= (1 << LCD_RW_pin);																  //    (RW = High, read)
																									  // port B, datove bity budu teraz input
	LCD_DATA_DDR &= ~((1 << LCD_D4_pin) | (1 << LCD_D5_pin) | (1 << LCD_D6_pin) | (1 << LCD_D7_pin)); //  Datove piny nastavime  na input, (Data pre disp)

	// Spravne by sa malo vycitavaj pred dobeznou hranou EN impulzu. Je tam urcity presah. Mozeme vycitat aj po dobeznej hrane.
	// Spravne vycitavanie pocas EN impulzu.
	EnOn();
	pom = PORT_DATA_RD_H; // vycitam High nibble AC
	EnOff();

	EnOn();
	pom |= PORT_DATA_RD_L; // vycitam Low nibble AC
	EnOff();
	// datove bity zase output
	LCD_DATA_DDR |= (1 << LCD_D4_pin) | (1 << LCD_D5_pin) | (1 << LCD_D6_pin) | (1 << LCD_D7_pin); // Datove piny nastavime na output (Data pre disp)
	// if(pom & 0x80) return 1;	// display je busy
	// else
	return pom; // display je not busy
}

void zob_text(char *s)
{
	register unsigned char c;
	while ((c = *s++))
		lcd_data(c); // retazec konci "nulou"
}

void def_znak(unsigned char *ZnakArray, unsigned char kam)
{
	lcd_command(0x40 | (kam << 3)); // nastavenie adresy znaku v CGRAM
	for (unsigned char i = 0; i < 8; i++)
		lcd_data(*(ZnakArray + i));
}

void lcd_putc(char ch)
{
	switch (ch)
	{
	case '\r':
		lcd_command(0xC0); // Presun na zaciatok 2. riadka (0x80 + 0x40)
		break;

	case '\f':
		lcd_command(0X80); // Presun na zaciatok 1. riadku
		break;

	default:
		lcd_data(ch);
	}
}

/* ********************************************************	*/
/* vypis retazca na poziciu, resp. podla nasledovnych 		*/
/* formatovacich prikazov                                       */
/*   : \r - prechod na novy riadok                              */
/*   : \f - prechod na zaciatok displeja                        */
/* ********************************************************     */

void lcd_puts(char *str)
{ /* definicia funkcie		*/

	while (*str)
	{
		lcd_putc(*str++);
	}
}

int8_t lcd_read_AC(void)
{ //  rd_BF
	char pom_AC;

	while ((pom_AC = busy_flag()) & 0x80)
		;
	// kedze po BF = 0 este cca 4us sa nezmenil obsah AC
	// treba vycitat este raz
	pom_AC = busy_flag();
	return pom_AC; // display not busy
}
